//////////////////////////////////////////////////////////////////////
//Sterne-Klasse, mit Rauten-Fundament. Hier werden alle Sterne-Shapes 
//verwaltet:Erzeugen, positionieren, phys.Eigenschaften definieren, zeichnen
//////////////////////////////////////////////////////////////////////
class Stern {
  //Einzig das Body-Objekt muss kontrolliert werden
  Body body;      
  color c;
  Vec2[] vertices1,vertices2,vertices3,vertices4;
  PolygonShape pS1,pS2,pS3,pS4;
  
  //Konstruktor
  Stern(float x, float y, color c) {
     this.c =c;
    // Body erzeugen
    erzeugeBody(new Vec2(x,y));
  }
  
  void entsorgen() {
    box2d.destroyBody(body);
  }
  
   boolean verschwunden() {
    Vec2 position = box2d.getBodyPixelCoord(body);
    // Unterhalb der Zeichenfläche?
    if (position.y > height) {
      entsorgen();
      return true;
    }
    return false;
  } 
  
  void erzeugeBody(Vec2 mittelPunkt){
     
    //Body-Definition
    BodyDef bD = new BodyDef();      
    bD.type = BodyType.DYNAMIC;  
    
    //Body-Position umrechnen
    bD.position.set(box2d.coordPixelsToWorld(mittelPunkt));   
    body = box2d.createBody(bD);  
    float faktor = random(0.4,1.5);
    //Vier Polygon-Shapes (Rauten) definieren
    pS1 = new PolygonShape();
    vertices1 = new Vec2[4];  
    vertices1[0] = box2d.vectorPixelsToWorld(new Vec2(faktor*20, 0));
    vertices1[1] = box2d.vectorPixelsToWorld(new Vec2(0, faktor*5));
    vertices1[2] = box2d.vectorPixelsToWorld(new Vec2(-faktor*20, 0));
    vertices1[3] = box2d.vectorPixelsToWorld(new Vec2(0, -faktor*5)); 
    pS1.set(vertices1, vertices1.length);
    
    pS2 = new PolygonShape();                
    vertices2 = new Vec2[4];
    vertices2[0] = box2d.vectorPixelsToWorld(new Vec2(faktor*5, 0));
    vertices2[1] = box2d.vectorPixelsToWorld(new Vec2(0, -faktor*20));
    vertices2[2] = box2d.vectorPixelsToWorld(new Vec2(-faktor*5, 0));
    vertices2[3] = box2d.vectorPixelsToWorld(new Vec2(0, faktor*20));  
    pS2.set(vertices2, vertices2.length);
    
    pS3 = new PolygonShape();
    vertices3 = new Vec2[4];  
    vertices3[0] = box2d.vectorPixelsToWorld(new Vec2(faktor*10, faktor*10));
    vertices3[1] = box2d.vectorPixelsToWorld(new Vec2(-faktor*2.5, faktor*2.5));
    vertices3[2] = box2d.vectorPixelsToWorld(new Vec2(-faktor*10, -faktor*10));
    vertices3[3] = box2d.vectorPixelsToWorld(new Vec2(faktor*2.5, -faktor*2.5)); 
    pS3.set(vertices3, vertices3.length);
    
    pS4 = new PolygonShape();                
    vertices4 = new Vec2[4];
    vertices4[0] = box2d.vectorPixelsToWorld(new Vec2(-faktor*10, faktor*10));
    vertices4[1] = box2d.vectorPixelsToWorld(new Vec2(-faktor*2.5, -faktor*2.5));
    vertices4[2] = box2d.vectorPixelsToWorld(new Vec2(faktor*10, -faktor*10));
    vertices4[3] = box2d.vectorPixelsToWorld(new Vec2(faktor*2.5, faktor*2.5));  
    pS4.set(vertices4, vertices4.length);
    
    
    //Ebene:  Masse proportional zu Durchmesser zum Quadrat
    body.createFixture(pS1,2.5*faktor*faktor);  
    body.createFixture(pS2,2.5*faktor*faktor);  
    body.createFixture(pS3,2.5*faktor*faktor);  
    body.createFixture(pS4,2.5*faktor*faktor);  
    float vX = random(-10,10);
    float vY = random(0,40);
    float vW = random(-30,30);
    body.setLinearVelocity(new Vec2(vX,vY));    
    body.setAngularVelocity(vW);
    
   
    
  }
   
  void display() {
    // Lage und Winkel besorgen
    Vec2 position = box2d.getBodyPixelCoord(body);		
    float winkel = -body.getAngle();//Minuszeichen ist wichtig! Sonst Überlagerung! 
    //Erzeugte Shapes verfügbar machen
    pS1.set(vertices1, vertices1.length);
    pS2.set(vertices2, vertices2.length);
    pS3.set(vertices3, vertices3.length);
    pS4.set(vertices4, vertices4.length);  
     
    rectMode(CENTER);
    pushMatrix();
    translate(position.x,position.y);		
   	rotate(winkel);  
    fill(c);
    noStroke();
    //Vier Shapes zeichnen 
    beginShape();
    // For every vertex, convert to pixel vector
    for (int i = 0; i < pS1.getVertexCount(); i++) {
      Vec2 v1 = box2d.vectorWorldToPixels(pS1.getVertex(i));
      vertex(v1.x, v1.y);
    }
    endShape(CLOSE);
    
    beginShape();
     // For every vertex, convert to pixel vector
    for (int i = 0; i < pS2.getVertexCount(); i++) {
      Vec2 v2 = box2d.vectorWorldToPixels(pS2.getVertex(i));
      vertex(v2.x, v2.y);
    }
    endShape(CLOSE);
    
    beginShape();
     // For every vertex, convert to pixel vector
    for (int i = 0; i < pS3.getVertexCount(); i++) {
      Vec2 v3 = box2d.vectorWorldToPixels(pS3.getVertex(i));
      vertex(v3.x, v3.y);
    }
    endShape(CLOSE);
    
    beginShape();
     // For every vertex, convert to pixel vector
    for (int i = 0; i < pS4.getVertexCount(); i++) {
      Vec2 v4 = box2d.vectorWorldToPixels(pS4.getVertex(i));
      vertex(v4.x, v4.y);
    }
    endShape(CLOSE);
    
    popMatrix();
  }
}